package MeasurementTools.MeasurementInfoTools;

import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import org.apache.commons.io.FileUtils;
import org.codehaus.jackson.map.ObjectMapper;
import org.codehaus.jackson.type.TypeReference;
import org.json.simple.JSONValue;

/** FileMeasurementInformation is a class to hold information created by analyzing datasets.
 * 
 * It stores the measure value and the name of the measure in a hash map for quick retrieval and storage.
 * 
 * It methods include reading a file containing the stored measures and saving the measures to a file, 
 * this is achieved with a JSON parser from Jackson codehaus @ http://jackson.codehaus.org/
 * 
 *
 * @author Matthew Connop
 * @version 4.4 May 2012 * 
 */
public final class MeasurementInformation {

    /** private Hashmap that stores dataset measure information key = measure name, value = float */
    private HashMap<String, Float> measures;

    /** Constructs a default FileMeasurementInformation that initializes the private Hashmap  
     */
    public MeasurementInformation(int[] inQIDs) {
        measures = new HashMap<>();
        StringBuilder string = new StringBuilder(0);
        for (int i=0; i<inQIDs.length;i++){
            String temp = Integer.toString(inQIDs[i]);
            string.append(temp);
        }        
        addMeasure("QIDS selected",Float.parseFloat(string.toString()));
        
    }
    
    protected MeasurementInformation(String fileLocation){
        try {
            readMeasures(fileLocation);
        } catch (IOException ex) {
        }
    }

    /** Adds a measure to the hashmap using the measure name as a key and the value of the measure
     * 
     * @param inMeasureName     the name of the measure to add 
     * @param inValue		The value of the measure
     */
    public void addMeasure(String inMeasureName, Float inValue) {
        measures.put(inMeasureName, inValue);
    }

    /** Retrieves the value associated to the measure name returns a null value if measure does not exist
     * 
     * @param inMeasureName	name of measure 
     * @return			Value of the measure
     */
    public Float getMeasure(String inMeasureName) {
        if (measures.containsKey(inMeasureName)) {
            return measures.get(inMeasureName);
        } else {
            return null;
        }
    }

    /** sets the value of a measure 
     * 
     * @param inMeasureName     the name of the measure to set 
     * @param inValue		The value of the measure
     */
    public void setMeasure(String inMeasureName, Float inValue) {
        if (measures.containsKey(inMeasureName)) {
            measures.put(inMeasureName, inValue);
        }
    }

    /** Retrieves the measures
     * 
     * @return             returns  HashMap<String, Float> of measures  
     */
    public HashMap<String, Float> getMeasurelist() {
        return measures;
    }

    /** Saves the measures to a file 
     * 
     * @param inFileName      The filename to save the measures list  
     * @throws IOException    throws an IOException
     */
    public void saveFile(String inFileName) throws IOException {
        String serialized = JSONValue.toJSONString(measures);
        FileUtils.writeStringToFile(new File(inFileName), serialized);
    }

    /** Reads the measures from a previously save file
     * 
     * @param inFileName      The filename to read the measures list
     * @throws IOException    throws an IOException
     */
    public void readMeasures(String inFileName) throws IOException {
        ObjectMapper mapper = new ObjectMapper();
        HashMap<String, Float> temp = mapper.readValue(
                new File(inFileName),
                new TypeReference<HashMap<String, Float>>() {
                });
        measures = temp;
    }

    /**
     * 
     * @param inMeasureName
     * @return 
     */
    public boolean hasMeasure(String inMeasureName) {
        if (measures.containsKey(inMeasureName)) {
            return true;
        } else {
            return false;
        }
    }
    
}
